@@ -8,49 +8,35 @@
 #' by [daemons()] and this function should not need to be invoked directly,
 #' unless deploying manually on remote resources.
 #'
-#' The network topology is such that daemons dial into the host or dispatcher,
-#' which listens at the `url` address. In this way, network resources may be
-#' added or removed dynamically and the host or dispatcher automatically
-#' distributes tasks to all available daemons.
+#' Daemons dial into the host or dispatcher, which listens at `url`. This
+#' allows network resources to be added or removed dynamically, with the host
+#' or dispatcher automatically distributing tasks to all connected daemons.
 #'
-#' @param url the character host or dispatcher URL to dial into, including the
-#'   port to connect to, e.g. 'tcp://hostname:5555' or
-#'   'tls+tcp://10.75.32.70:5555'.
-#' @param dispatcher logical value, which should be set to TRUE if using
-#'   dispatcher and FALSE otherwise.
-#' @param ... reserved, but not currently used.
-#' @param asyncdial whether to perform dials asynchronously. The default FALSE
-#'   will error if a connection is not immediately possible (for instance if
-#'   [daemons()] has yet to be called on the host, or the specified port is not
-#'   open etc.). Specifying TRUE continues retrying (indefinitely) if not
-#'   immediately successful, which is more resilient but can mask potential
-#'   connection issues.
-#' @param autoexit logical value, whether the daemon should exit automatically
-#'   when its socket connection ends. By default, the process ends immediately
-#'   when the host process ends. Supply `NA` to have a daemon complete any tasks
-#'   in progress before exiting (see 'Persistence' section below).
-#' @param cleanup logical value, whether to perform cleanup of the global
-#'   environment and restore attached packages and options to an initial state
-#'   after each evaluation.
-#' @param output logical value, to output generated stdout / stderr if TRUE, or
-#'   else discard if FALSE. Specify as TRUE in the `...` argument to [daemons()]
-#'   or [launch_local()] to provide redirection of output to the host process
-#'   (applicable only for local daemons).
-#' @param idletime integer milliseconds maximum time to wait for a task (idle
-#'   time) before exiting.
-#' @param walltime integer milliseconds soft walltime (time limit) i.e. the
-#'   minimum amount of real time elapsed before exiting.
-#' @param maxtasks integer maximum number of tasks to execute (task limit)
-#'   before exiting.
-#' @param tlscert required for secure TLS connections over 'tls+tcp://'.
-#'   **Either** the character path to a file containing X.509 certificate(s) in
-#'   PEM format, comprising the certificate authority certificate chain starting
-#'   with the TLS certificate and ending with the CA certificate, **or** a
-#'   length 2 character vector comprising (i) the certificate authority
-#'   certificate chain and (ii) the empty string `""`.
-#' @param rs the initial value of .Random.seed. This is set automatically using
-#'   L'Ecuyer-CMRG RNG streams generated by the host process if applicable, and
-#'   should not be independently supplied.
+#' @param url (character) host or dispatcher URL to dial into, e.g.
+#'   'tcp://hostname:5555' or 'tls+tcp://10.75.32.70:5555'.
+#' @param dispatcher (logical) whether dialing into dispatcher or directly to
+#'   host.
+#' @param ... reserved for future use.
+#' @param asyncdial (logical) whether to dial asynchronously. `FALSE` errors if
+#'   connection fails immediately. `TRUE` retries indefinitely (more resilient
+#'   but can mask connection issues).
+#' @param autoexit (logical) whether to exit when the socket connection ends.
+#'   `TRUE` exits immediately, `NA` completes current task first, `FALSE`
+#'   persists indefinitely. See Persistence section.
+#' @param cleanup (logical) whether to restore global environment, packages, and
+#'   options to initial state after each evaluation.
+#' @param output (logical) whether to output stdout/stderr. For local daemons
+#'   via [daemons()] or [launch_local()], redirects output to host process.
+#' @param idletime (integer) milliseconds to wait idle before exiting.
+#' @param walltime (integer) milliseconds of real time before exiting (soft
+#'   limit).
+#' @param maxtasks (integer) maximum tasks to execute before exiting.
+#' @param tlscert (character) for secure TLS connections. Either a file path to
+#'   PEM-encoded certificate authority certificate chain (starting with the TLS
+#'   certificate and ending with the CA certificate), or a length-2 vector of
+#'   (certificate chain, empty string `""`).
+#' @param rs (integer vector) initial `.Random.seed` value. Set automatically by
+#'   host process; do not supply manually.
 #'
 #' @return Invisibly, an integer exit code: 0L for normal termination, and a
 #'   positive value if a self-imposed limit was reached: 1L (idletime), 2L
@@ -133,6 +119,8 @@ daemon <- function(
         aio <- recv_aio(sock, mode = 1L, timeout = timeout, cv = cv)
         wait(cv) || break
         m <- collect_aio(aio)
+        # handle cancellation received late: raw (if deserialization failed)
+        is.raw(m) && next
         is.integer(m) &&
           {
             m == 5L || next
@@ -218,28 +206,29 @@ daemon <- function(
 # internals --------------------------------------------------------------------
 
 eval_mirai <- function(._mirai_., sock = NULL) {
-  withRestarts(
+  if (length(sock)) {
+    cancel <- recv_aio(sock, mode = 8L, cv = substitute())
+    on.exit(stop_aio(cancel))
+  }
+  tryCatch(
     withCallingHandlers(
       {
-        if (length(sock)) {
-          cancel <- recv_aio(sock, mode = 8L, cv = substitute())
-          on.exit(stop_aio(cancel))
-        }
         list2env(._mirai_.[["._globals_."]], envir = globalenv())
         sock <- otel_eval_span(._mirai_.[["._otel_."]])
         eval(._mirai_.[["._expr_."]], envir = ._mirai_., enclos = globalenv())
       },
       error = function(cnd) {
-        otel_set_span_error(sock, "miraiError")
-        invokeRestart("mirai_error", cnd, sys.calls())
-      },
-      interrupt = function(cnd) {
-        otel_set_span_error(sock, "miraiInterrupt")
-        invokeRestart("mirai_interrupt")
+        `[[<-`(., "syscalls", sys.calls())
       }
     ),
-    mirai_error = mk_mirai_error,
-    mirai_interrupt = mk_interrupt_error
+    error = function(cnd) {
+      otel_set_span_error(sock, "miraiError")
+      mk_mirai_error(cnd)
+    },
+    interrupt = function(cnd) {
+      otel_set_span_error(sock, "miraiInterrupt")
+      mk_mirai_interrupt()
+    }
   )
 }
 
